package com.fzm.common.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fzm.common.entity.Order;
import com.fzm.common.entity.dto.OrderDto;
import com.fzm.common.entity.dto.OrderProcessMsg;
import com.fzm.common.enums.OrderStatus;
import com.fzm.common.enums.PayScene;
import com.fzm.common.mapper.OrderMapper;
import com.fzm.common.service.CopyrightApplyService;
import com.fzm.common.service.NftService;
import com.fzm.common.service.OrderService;
import com.fzm.common.utils.JwtUtil;
import com.fzm.common.utils.SnowflakeUtil;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.List;

/**
 * @author tangtuo
 * @date 2022/1/19 15:42
 */
@Service
@Transactional(rollbackFor = RuntimeException.class)
public class OrderServiceImpl extends ServiceImpl<OrderMapper, Order> implements OrderService {

    @Resource
    private NftService nftService;

    @Resource
    private CopyrightApplyService copyrightApplyService;

    @Resource
    private SnowflakeUtil snowflakeUtil;

    @Resource
    private HttpServletRequest request;

    @Resource
    private RabbitTemplate rabbitTemplate;

    @Override
    public Order getByPaySceneAndProductId(Integer payScene, Integer productId, Integer orderStatus) {
        QueryWrapper<Order> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("pay_scene", payScene)
                .eq("product_id", productId)
                .eq("order_status", orderStatus);
        return this.getOne(queryWrapper);
    }

    @Override
    public void updateOrderStatus(Long out_trade_no, OrderStatus orderStatus) {
        Order order = new Order();
        order.setId(out_trade_no);
        order.setOrderStatus(orderStatus.getStatus());
        this.updateById(order);
    }

    @Override
    public Long createOrder(OrderDto orderDto) {
        Order order = new Order();
        long id = snowflakeUtil.snowflakeId();
        Integer productId = orderDto.getProductId();
        String orderName;
        Integer payScene = orderDto.getPayScene();
        if (PayScene.NFT.getCode().equals(payScene)) {
            orderName = nftService.getById(productId).getName();
        } else {
            orderName = copyrightApplyService.getById(productId).getOpusName();
        }
        order.setId(id);
        order.setOrderName(orderName);
        order.setPayScene(payScene);
        order.setOrderName(orderName);
        order.setFee(orderDto.getFee());
        order.setProductId(productId);
        order.setUserId(JwtUtil.getUserIdFromToken(request.getHeader("Authorization")));
        Integer orderStatus;
        if (orderDto.getFee() > 0) {
            orderStatus = OrderStatus.PAYING.getStatus();
        } else {
            // 订单金额为0的话,创建订单时,状态为支付成功
            orderStatus = OrderStatus.PAYED.getStatus();
            processOrder(id, productId, payScene);
        }
        order.setOrderStatus(orderStatus);
        this.save(order);
        return id;
    }

    @Override
    public Boolean updatePayType(Long orderId, Integer payType) {
        Order o = new Order();
        o.setId(orderId);
        o.setPayType(payType);
        return updateById(o);
    }

    @Override
    public Boolean cancel(Long orderId, OrderStatus orderStatus) {
        Order order = this.getById(orderId);
        Integer productId = order.getProductId();
        // 判断订单的类型
        if (PayScene.NFT.getCode().equals(order.getPayScene())) {
            // 删除nft的相关信息
            nftService.delete(productId);
        } else {
            // 删除版权申请的相关信息
            copyrightApplyService.delete(productId);
        }
        // 修改订单状态为取消
        this.updateOrderStatus(orderId, orderStatus);
        return true;
    }

    @Override
    public List<Order> listOrder(Integer payScene) {
        Integer userId = JwtUtil.getUserIdFromToken(request.getHeader("Authorization"));
        QueryWrapper<Order> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("pay_scene", payScene);
        queryWrapper.eq("user_id", userId);
        return this.list(queryWrapper);
    }

    @Override
    public void processOrder(Long orderId, Integer productId, Integer payScene) {
        // 订单支付成功后,给mq发送一条消息,处理nft发行或版权申请的状态
        OrderProcessMsg orderProcessMsg = new OrderProcessMsg(orderId, productId, payScene);
        rabbitTemplate.convertAndSend("order-exchange", "order.process", orderProcessMsg);
    }
}
